##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Test 'soap' ZCML Namespace directives.

$Id: test_directives.py 92902 2008-11-13 12:18:51Z jfroche $
"""

from zope.configuration import xmlconfig
from zope.configuration.exceptions import ConfigurationError
from zope.app.component.tests.views import IC
from zope.component import getMultiAdapter, queryMultiAdapter
from zope.app.testing.placelesssetup import PlacelessSetup
from z3c.soap.interfaces import ISOAPRequest
from zope.interface import alsoProvides
from zope.interface import implements
import unittest
import z3c.soap.tests
from zope.publisher.browser import TestRequest

request = TestRequest(ISOAPRequest)
alsoProvides(request, ISOAPRequest)


class Ob(object):

    implements(IC)

ob = Ob()


class DirectivesTest(PlacelessSetup, unittest.TestCase):

    def testView(self):
        self.assertEqual(
            queryMultiAdapter((ob, request), name='test'), None)
        xmlconfig.file("soap.zcml", z3c.soap.tests)
        self.assertEqual(
            str(queryMultiAdapter((ob, request), name='test').__class__),
            "<class 'Products.Five.metaclass.V1'>")

    def testInterfaceAndAttributeProtectedView(self):
        xmlconfig.file("soap.zcml", z3c.soap.tests)
        v = getMultiAdapter((ob, request), name='test4')
        self.assertEqual(v.index(), 'V1 here')
        self.assertEqual(v.action(), 'done')

    def testDuplicatedInterfaceAndAttributeProtectedView(self):
        xmlconfig.file("soap.zcml", z3c.soap.tests)
        v = getMultiAdapter((ob, request), name='test5')
        self.assertEqual(v.index(), 'V1 here')
        self.assertEqual(v.action(), 'done')

    def testIncompleteProtectedViewNoPermission(self):
        self.assertRaises(ConfigurationError, xmlconfig.file,
                          "soap_error.zcml", z3c.soap.tests)

    def test_no_name(self):
        xmlconfig.file("soap.zcml", z3c.soap.tests)
        v = getMultiAdapter((ob, request), name='index')
        self.assertEqual(v(), 'V1 here')
        v = getMultiAdapter((ob, request), name='action')
        self.assertEqual(v(), 'done')


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(DirectivesTest),
        ))

if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
SOAP configuration code

$Id: metaconfigure.py 92927 2008-11-14 15:37:11Z jfroche $
"""

from zope.interface import Interface
from zope.security.checker import CheckerPublic
from zope.component.interface import provideInterface
from Products.Five.security import protectClass, protectName
from zope.app.publisher.browser.viewmeta import _handle_for

# XXX handler is non-public.  Should call directives instead
from zope.app.component.metaconfigure import handler

from inspect import ismethod
from interfaces import ISOAPRequest
from Globals import InitializeClass as initializeClass
from Products.Five.security import getSecurityInfo
from Products.Five.metaclass import makeClass

from Products.Five.browser import BrowserView
from Products.Five.browser.metaconfigure import ViewMixinForAttributes
from Products.Five.security import CheckerPrivateId


def view(_context, for_=None, interface=None, methods=None,
         class_=None, permission=None, name=None):

    interface = interface or []
    methods = methods or []

    # If there were special permission settings provided, then use them
    if permission == 'zope.Public':
        permission = CheckerPublic

    require = {}
    for attr_name in methods:
        require[attr_name] = permission

    if interface:
        for iface in interface:
            for field_name in iface:
                require[field_name] = permission
            _context.action(
                discriminator = None,
                callable = provideInterface,
                args = ('', for_))

    cdict = getSecurityInfo(class_)

    if name:
        cdict['__name__'] = name
        new_class = makeClass(class_.__name__,
                              (class_, BrowserView), cdict)

        _handle_for(_context, for_)
        # Register the new view.
        _context.action(
            discriminator = ('view', (for_, ), name, ISOAPRequest),
            callable = handler,
            args = ('registerAdapter',
                    new_class, (for_, ISOAPRequest),
                    Interface, name,
                    _context.info)
            )
        _context.action(
            discriminator = ('five:protectClass', new_class),
            callable = protectClass,
            args = (new_class, permission))

        for name in require:
            _context.action(
                discriminator = ('five:protectName', new_class, name),
                callable = protectName,
                args = (new_class, name, permission))

        #else its private:
        allowed = require
        private_attrs = [name for name in dir(new_class)
                         if (not name.startswith('_')) and
                            (name not in allowed) and
                            ismethod(getattr(new_class, name))]
        for attr in private_attrs:
            _context.action(
                discriminator = ('five:protectName', new_class, attr),
                callable = protectName,
                args = (new_class, attr, CheckerPrivateId))

    else:
        for name in require:
            cdict.update({'__page_attribute__': name,
                          '__name__': name})
            new_class = makeClass(class_.__name__,
                                  (class_, ViewMixinForAttributes),
                                  cdict)

            func = getattr(new_class, name)
            if not func.__doc__:
                # cannot test for MethodType/UnboundMethod here
                # because of ExtensionClass
                if hasattr(func, 'im_func'):
                    # you can only set a docstring on functions, not
                    # on method objects
                    func = func.im_func
                func.__doc__ = "Stub docstring to make ZPublisher work"

            _context.action(
                discriminator = ('view', (for_, ), name, ISOAPRequest),
                callable = handler,
                args = ('registerAdapter',
                        new_class,
                        (for_, ISOAPRequest), Interface, name,
                        _context.info))

            _context.action(
                discriminator = ('five:protectClass', new_class),
                callable = protectClass,
                args = (new_class, permission))

            _context.action(
                discriminator = ('five:protectName', new_class, name),
                callable = protectName,
                args = (new_class, name, permission))

            _context.action(
                discriminator = ('five:initialize:class', new_class),
                callable = initializeClass,
                args = (new_class, )
                )

    # Register the used interfaces with the interface service
    if for_ is not None:
        _context.action(
            discriminator = None,
            callable = provideInterface,
            args = ('', for_))



##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
SOAP Publication Tests

$Id: test_soappublication.py 92902 2008-11-13 12:18:51Z jfroche $
"""
import unittest

from zope.app.publication.tests.test_zopepublication import \
     BasePublicationTests
from zope.app.publication.traversers import TestTraverser
from zope.app.publication.soap import SOAPPublication
from zope.interface import Interface, implements
from zope.proxy import removeAllProxies
from zope.publisher.interfaces import NotFound
from z3c.soap.interfaces import ISOAPPresentation, ISOAPRequest
from z3c.soap.interfaces import ISOAPPublisher
from z3c.soap.publisher import SOAPRequest
from zope.app.testing import ztapi
from StringIO import StringIO


class TestRequest(SOAPRequest):

    def __init__(self, body_instream=None, environ=None,
                 response=None, **kw):

        _testEnv = {
            'SERVER_URL': 'http://127.0.0.1',
            'HTTP_HOST': '127.0.0.1',
            'CONTENT_LENGTH': '0',
            'GATEWAY_INTERFACE': 'TestFooInterface/1.0'}

        if environ:
            _testEnv.update(environ)
        if kw:
            _testEnv.update(kw)
        if body_instream is None:
            body_instream = StringIO('')

        super(TestRequest, self).__init__(
            body_instream, _testEnv, response)


class SimpleObject(object):

    def __init__(self, v):
        self.v = v


class SOAPPublicationTests(BasePublicationTests):

    klass = SOAPPublication

    def _createRequest(self, path, publication, **kw):
        request = TestRequest(PATH_INFO=path, **kw)
        request.setPublication(publication)
        return request

    def testTraverseName(self):
        pub = self.klass(self.db)

        class C(object):
            x = SimpleObject(1)
        ob = C()
        r = self._createRequest('/x', pub)
        ztapi.provideView(None, ISOAPRequest, ISOAPPublisher,
                          '', TestTraverser)
        ob2 = pub.traverseName(r, ob, 'x')
        self.assertEqual(removeAllProxies(ob2).v, 1)

    def testDenyDirectMethodAccess(self):
        pub = self.klass(self.db)

        class I(Interface):
            pass

        class C(object):
            implements(I)

            def foo(self):
                return 'bar'

        class V(object):

            def __init__(self, context, request):
                pass
            implements(ISOAPPresentation)

        ob = C()
        r = self._createRequest('/foo', pub)

        ztapi.provideView(I, ISOAPPresentation, Interface, 'view', V)
        ztapi.setDefaultViewName(I, 'view', type=ISOAPPresentation)
        self.assertRaises(NotFound, pub.traverseName, r, ob, 'foo')

    def testTraverseNameView(self):
        pub = self.klass(self.db)

        class I(Interface):
            pass

        class C(object):
            implements(I)

        ob = C()

        class V(object):

            def __init__(self, context, request):
                pass
            implements(ISOAPPresentation)


        # Register the simple traverser so we can traverse without @@
        from z3c.soap.interfaces import ISOAPPublisher, ISOAPRequest
        from zope.app.publication.traversers import SimpleComponentTraverser
        ztapi.provideView(Interface, ISOAPRequest, ISOAPPublisher, '',
                          SimpleComponentTraverser)

        r = self._createRequest('/@@spam', pub)
        ztapi.provideView(I, ISOAPRequest, Interface, 'spam', V)
        ob2 = pub.traverseName(r, ob, '@@spam')
        self.assertEqual(removeAllProxies(ob2).__class__, V)

        ob2 = pub.traverseName(r, ob, 'spam')
        self.assertEqual(removeAllProxies(ob2).__class__, V)

    def testTraverseNameServices(self):
        pub = self.klass(self.db)

        class C(object):

            def getSiteManager(self):
                return SimpleObject(1)
        ob = C()
        r = self._createRequest('/++etc++site', pub)
        ob2 = pub.traverseName(r, ob, '++etc++site')
        self.assertEqual(removeAllProxies(ob2).v, 1)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SOAPPublicationTests),
        ))

if __name__ == '__main__':
    unittest.main()



"""SOAP support module

Based on the XML-RPC Zope support module written by Eric Kidd at UserLand
software, with much help from Jim Fulton at DC.

This code hooks Zope up to Fredrik Lundh's Python SOAP library.
"""

import sys
import types
from string import replace
from zExceptions import Unauthorized
from zope.publisher.xmlrpc import premarshal
from ZSI import TC, ParsedSoap
from ZSI import SoapWriter, Fault
from zope.component import queryUtility
from z3c.soap.interfaces import IZSIRequestType, IZSIResponseType
import ZSI
import logging
import traceback
from zope.interface import implements
from z3c.soap.interfaces import ISOAPResponse


class SOAPParser(object):

    def __init__(self, data):
        self.parsed = ParsedSoap(data)
        self.root = self.parsed.body_root
        self.target = self.root.localName
        self.method = replace(self.target, '.', '/')

    def parse(self):
        data = ZSI._child_elements(self.root)
        if len(data) == 0:
            params = ()
        else:
            resolver = queryUtility(IZSIRequestType, name=self.target)
            if resolver is None:
                targetWithNamespace = "%s/%s" % (self.root.namespaceURI,
                                                 self.target)
                resolver = queryUtility(IZSIRequestType,
                                        name=targetWithNamespace)
            if resolver and hasattr(resolver, 'typecode'):
                tc = resolver.typecode
                params = [resolver.typecode.parse(self.root, self.parsed)]

                resolver = queryUtility(IZSIResponseType, name=self.target)
                if resolver is None:
                    targetWithNamespace = "%s/%s" % (self.root.namespaceURI,
                                                 self.target)

                    resolver = queryUtility(IZSIResponseType,
                                            name=targetWithNamespace)
                params.append(resolver)
            else:
                tc = TC.Any()
                params = [tc.parse(e, self.parsed) for e in data]
            params = tuple(params)
        return params


def parse_input(data):
    parser = SOAPParser(data)
    return parser.parse()


class SOAPResponse:
    """Customized Response that handles SOAP-specific details.

    We override setBody to marhsall Python objects into SOAP. We
    also override exception to convert errors to SOAP faults.

    If these methods stop getting called, make sure that ZPublisher is
    using the soap.Response object created above and not the original
    HTTPResponse object from which it was cloned.

    It's probably possible to improve the 'exception' method quite a bit.
    The current implementation, however, should suffice for now.
    """
    implements(ISOAPResponse)
    _contentType = 'text/xml'

    _soap11 = None
    _soap12 = None
    # Because we can't predict what kind of thing we're customizing,
    # we have to use delegation, rather than inheritence to do the
    # customization.

    def __init__(self, real):
        self.__dict__['_real']=real

    def __getattr__(self, name):
        return getattr(self._real, name)

    def __setattr__(self, name, v):
        return setattr(self._real, name, v)

    def __delattr__(self, name):
        return delattr(self._real, name)

    def setBody(self, body, title='', is_error=0, bogus_str_search=None):
        if isinstance(body, Fault):
            # Convert Fault object to SOAP response.
            body = body.AsSOAP()
        else:
            # Marshall our body as an SOAP response. Strings will be sent
            # strings, integers as integers, etc. We do *not* convert
            # everything to a string first.
            try:
                target = self._method
                body = premarshal(body)
                result = body
                if hasattr(result, 'typecode'):
                    tc = result.typecode
                else:
                    tc = TC.Any(aslist=1, pname=target + 'Response')
                    result = [result]
                sw = SoapWriter(nsdict={}, header=True, outputclass=None,
                        encodingStyle=None)
                body = str(sw.serialize(result, tc))
            except:
                self.exception()
                return

        # Set our body to the message, and fix our MIME type.
        self._real.setBody(body)
        self._setHeader()
        return self

    def exception(self, fatal=0, info=None,
                  absuri_match=None, tag_search=None):
        if isinstance(info, tuple) and len(info)==3:
            t, v, tb = info
        else:
            t, v, tb = sys.exc_info()

        content = "".join(traceback.format_tb(tb))
        logger = logging.getLogger('Zope')
        logger.info('SOAPException: %s' % content)
        f=v
        if t == 'Unauthorized' or t == Unauthorized or (
           isinstance(t, types.ClassType) and issubclass(t, Unauthorized)):
            self._real.setStatus(401)
            f = ZSI.Fault(Fault.Server, "Not authorized")
        elif not isinstance(v, Fault):
            self._real.setStatus(500)
            f = ZSI.FaultFromException(u"%s : %s" % (v, content), 0)
        self.setBody(f)
        return tb

    def _setHeader(self):
        self.setHeader('content-length', len(self._real.body))
        self._real.setHeader('content-type', self._contentType)
        if self._soap11:
            self._real.setHeader('content-type', 'text/xml')
        if self._soap12:
            self._real.setHeader('content-type', 'application/soap+xml')


response=SOAPResponse



from ZSI import TCcompound, TC
from ZSI.schema import LocalElementDeclaration, ElementDeclaration
from ZSI.schema import GED


class ns0:
    targetNamespace = "urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2"


class ns1:
    targetNamespace = "urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2:Messages"

    class ValidateEmailRequest_Dec(TCcompound.ComplexType, ElementDeclaration):
        literal = "ValidateEmailRequest"
        schema = "urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2:Messages"

        def __init__(self, **kw):
            ns = ns1.ValidateEmailRequest_Dec.schema
            TClist = [TC.String(pname=(ns, "Email"), aname="_Email",
                                minOccurs=1, maxOccurs=1, nillable=False,
                                typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2:Messages', u'ValidateEmailRequest')
            kw["aname"] = "_ValidateEmailRequest"
            self.attribute_typecode_dict = {}
            TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)

            class Holder:
                typecode = self

                def __init__(self):
                    # pyclass
                    self._Email = None
                    return
            Holder.__name__ = "ValidateEmailRequest_Holder"
            self.pyclass = Holder


    class ValidateEmailResponse_Dec(TCcompound.ComplexType, ElementDeclaration):
        literal = "ValidateEmailResponse"
        schema = "urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2:Messages"

        def __init__(self, **kw):
            ns = ns1.ValidateEmailResponse_Dec.schema
            TClist = [self.__class__.Status_Dec(minOccurs=1, maxOccurs=1,
                                                nillable=False,
                                                encoded=kw.get("encoded"))]
            kw["pname"] = (u'urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2:Messages', u'ValidateEmailResponse')
            kw["aname"] = "_ValidateEmailResponse"
            self.attribute_typecode_dict = {}
            TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)

            class Holder:
                typecode = self

                def __init__(self):
                    # pyclass
                    self._Status = None
                    return

            Holder.__name__ = "ValidateEmailResponse_Holder"
            self.pyclass = Holder

        class Status_Dec(TC.String, LocalElementDeclaration):
            literal = "Status"
            schema = "urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2:Messages"

            def __init__(self, **kw):
                kw["pname"] = "Status"
                kw["aname"] = "_Status"
                TC.String.__init__(self, **kw)

                class IHolder(str):
                    typecode=self
                self.pyclass = IHolder
                IHolder.__name__ = "_Status_immutable_holder"

validateEmailIn = GED("urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2:Messages", "ValidateEmailRequest").pyclass
validateEmailOut = GED("urn:ws-xwebservices-com:XWebEmailValidation:EmailValidation:v2:Messages", "ValidateEmailResponse").pyclass
